# This file was auto-generated by Fern from our API Definition.

from .slim_composio_tool_definition import SlimComposioToolDefinition

ComponentsSchemasSlimComposioToolDefinition = SlimComposioToolDefinition
