# This file was auto-generated by Fern from our API Definition.

import typing

ProcessingFailureReasonEnum = typing.Union[
    typing.Literal["EXCEEDED_CHARACTER_LIMIT", "INVALID_FILE", "INVALID_CREDENTIALS"], typing.Any
]
