# WARNING: This file will be removed in a future release. Please import from "vellum.client" instead.

from vellum.client.types.basic_vectorizer_sentence_transformers_multi_qa_mpnet_base_dot_v_1_request import *
