# This file was auto-generated by Fern from our API Definition.

import typing

IndexingStateEnum = typing.Union[
    typing.Literal["AWAITING_PROCESSING", "QUEUED", "INDEXING", "INDEXED", "FAILED"], typing.Any
]
