# This file was auto-generated by Fern from our API Definition.

import typing

from .array_input import ArrayInput
from .audio_input import AudioInput
from .chat_history_input import ChatHistoryInput
from .document_input import DocumentInput
from .error_input import ErrorInput
from .function_call_input import FunctionCallInput
from .image_input import ImageInput
from .json_input import JsonInput
from .number_input import NumberInput
from .search_results_input import SearchResultsInput
from .string_input import StringInput
from .video_input import VideoInput

WorkflowInput = typing.Union[
    StringInput,
    JsonInput,
    ChatHistoryInput,
    NumberInput,
    SearchResultsInput,
    ErrorInput,
    ArrayInput,
    FunctionCallInput,
    AudioInput,
    VideoInput,
    ImageInput,
    DocumentInput,
]
