# This file was auto-generated by Fern from our API Definition.

from .composio_integration_exec_config import ComposioIntegrationExecConfig

ComponentsSchemasComposioIntegrationExecConfig = ComposioIntegrationExecConfig
