# This file was auto-generated by Fern from our API Definition.

import typing

WorkflowExecutionUsageCalculationErrorCodeEnum = typing.Union[
    typing.Literal["UNKNOWN", "DEPENDENCIES_FAILED", "NO_USAGE_CALCULATED", "INTERNAL_SERVER_ERROR"], typing.Any
]
