# This file was auto-generated by Fern from our API Definition.

import typing

WorkflowExecutionEventErrorCode = typing.Union[
    typing.Literal[
        "WORKFLOW_INITIALIZATION",
        "WORKFLOW_CANCELLED",
        "WORKFLOW_TIMEOUT",
        "PROVIDER_CREDENTIALS_UNAVAILABLE",
        "INTEGRATION_CREDENTIALS_UNAVAILABLE",
        "NODE_EXECUTION_COUNT_LIMIT_REACHED",
        "INTERNAL_SERVER_ERROR",
        "NODE_EXECUTION",
        "NODE_CANCELLED",
        "NODE_TIMEOUT",
        "LLM_PROVIDER",
        "INVALID_TEMPLATE",
        "INVALID_INPUTS",
        "PROVIDER_QUOTA_EXCEEDED",
        "USER_DEFINED_ERROR",
    ],
    typing.Any,
]
