# This file was auto-generated by Fern from our API Definition.

import typing

from .workflow_event import WorkflowEvent

CreateWorkflowEventRequest = typing.Union[typing.List[WorkflowEvent], WorkflowEvent]
